@@ -63,6 +63,8 @@ module Database.DuckDB.Simple (
     ToRow (..),
     FieldBinding,
     NamedParam (..),
+    DuckDBColumnType (..),
+    duckdbColumnType,
     Null (..),
     Only (..),
     (:.) (..),
@@ -73,11 +75,11 @@ module Database.DuckDB.Simple (
     deleteFunction,
 ) where
 
-import Control.Monad (forM, foldM, join, void, when, zipWithM, zipWithM_)
+import Control.Monad (forM, join, void, when, zipWithM, zipWithM_)
 import Data.IORef (IORef, atomicModifyIORef', mkWeakIORef, newIORef, readIORef, writeIORef)
 import Control.Exception (SomeException, bracket, finally, mask, onException, throwIO, try)
 import qualified Data.ByteString as BS
-import Data.Bits ((.|.), shiftL)
+import Data.Bits ((.|.), shiftL, Bits (..))
 import Data.Int (Int16, Int32, Int64, Int8)
 import Data.Maybe (isJust, isNothing)
 import Data.Text (Text)
@@ -105,9 +107,10 @@ import Database.DuckDB.Simple.FromField
     , FieldValue (..)
     , FieldParser
     , FromField (..)
-    , ResultError (..)
     , IntervalValue (..)
+    , ResultError (..)
     , TimeWithZone (..)
+    , fromBigNumBytes
     )
 import Database.DuckDB.Simple.FromRow
     ( FromRow (..)
@@ -135,15 +138,16 @@ import Database.DuckDB.Simple.Internal
     , withQueryCString
     , withStatementHandle
     )
-import Database.DuckDB.Simple.ToField (FieldBinding, NamedParam (..), ToField (..), bindFieldBinding, renderFieldBinding)
+import Database.DuckDB.Simple.ToField (DuckDBColumnType (..), FieldBinding, NamedParam (..), ToField (..), bindFieldBinding, duckdbColumnType, renderFieldBinding)
 import Database.DuckDB.Simple.ToRow (ToRow (..))
 import Database.DuckDB.Simple.Types (FormatError (..), Null (..), Only (..), (:.) (..))
 import Foreign.C.String (CString, peekCString, withCString)
-import Foreign.C.Types (CBool (..), CDouble (..))
+import Foreign.C.Types (CBool (..))
 import Foreign.Marshal.Alloc (alloca, free, malloc)
 import Foreign.Ptr (Ptr, castPtr, nullPtr, plusPtr)
-import Foreign.Storable (peek, peekElemOff, poke)
+import Foreign.Storable (Storable (..), peekElemOff, peekByteOff, pokeByteOff)
 import Database.DuckDB.Simple.Ok (Ok(..))
+import qualified Data.UUID as UUID
 
 -- | Open a DuckDB database located at the supplied path.
 open :: FilePath -> IO Connection
@@ -537,7 +541,7 @@ startStatementStream stmt =
     withStatementHandle stmt \handle -> do
         columns <- collectStreamColumns handle
         resultPtr <- malloc
-        rc <- c_duckdb_execute_prepared_streaming handle resultPtr
+        rc <- c_duckdb_execute_prepared handle resultPtr
         if rc /= DuckDBSuccess
             then do
                 (errMsg, errType) <- fetchResultError resultPtr
@@ -582,7 +586,7 @@ streamNextRow queryText stream@StatementStream{statementStreamChunk = Just chunk
 
 fetchChunk :: StatementStream -> IO StatementStream
 fetchChunk stream@StatementStream{statementStreamResult} = do
-    chunk <- c_duckdb_stream_fetch_chunk statementStreamResult
+    chunk <- c_duckdb_fetch_chunk statementStreamResult
     if chunk == nullPtr
         then pure stream
         else do
@@ -638,7 +642,7 @@ buildRow queryText columns vectors rowIdx =
     zipWithM (buildField queryText rowIdx) columns vectors
 
 buildField :: Query -> Int -> StatementStreamColumn -> StatementStreamChunkVector -> IO Field
-buildField queryText rowIdx column StatementStreamChunkVector{statementStreamChunkVectorData, statementStreamChunkVectorValidity} = do
+buildField queryText rowIdx column StatementStreamChunkVector{statementStreamChunkVectorHandle, statementStreamChunkVectorData, statementStreamChunkVectorValidity} = do
     let duckIdx = fromIntegral rowIdx
         dtype = statementStreamColumnType column
     valid <- chunkIsRowValid statementStreamChunkVectorValidity duckIdx
@@ -695,8 +699,16 @@ buildField queryText rowIdx column StatementStreamChunkVector{statementStreamChu
                     raw <- peekElemOff (castPtr statementStreamChunkVectorData :: Ptr Double) rowIdx
                     pure (FieldDouble raw)
                 DuckDBTypeVarchar -> FieldText <$> chunkDecodeText statementStreamChunkVectorData duckIdx
-                DuckDBTypeHugeInt -> error "duckdb-simple: HUGEINT is not supported"
-                DuckDBTypeUHugeInt -> error "duckdb-simple: HUGEINT is not supported"
+                DuckDBTypeList -> FieldList <$> decodeListElements statementStreamChunkVectorHandle statementStreamChunkVectorData rowIdx
+                DuckDBTypeMap -> FieldMap <$> decodeMapPairs statementStreamChunkVectorHandle statementStreamChunkVectorData rowIdx
+                DuckDBTypeStruct -> error "duckdb-simple: STRUCT columns are not supported"
+                DuckDBTypeUnion -> error "duckdb-simple: UNION columns are not supported"
+                DuckDBTypeHugeInt -> do
+                    raw <- peekElemOff (castPtr statementStreamChunkVectorData :: Ptr DuckDBHugeInt) rowIdx
+                    pure (FieldHugeInt (duckDBHugeIntToInteger raw))
+                DuckDBTypeUHugeInt -> do
+                    raw <- peekElemOff (castPtr statementStreamChunkVectorData :: Ptr DuckDBUHugeInt) rowIdx
+                    pure (FieldUHugeInt (duckDBUHugeIntToInteger raw))
                 _ ->
                     throwIO (streamingUnsupportedTypeError queryText column)
     pure
@@ -1014,19 +1026,32 @@ convertRowsWith parser queryText rows =
 collectRows :: Ptr DuckDBResult -> IO [[Field]]
 collectRows resPtr = do
     columns <- collectResultColumns resPtr
-    chunkCount <- (fromIntegral <$> c_duckdb_result_chunk_count resPtr) :: IO Int
-    if null columns || chunkCount <= 0
-        then pure []
-        else do
-            (_, chunks) <-
-                foldM
-                    (\(rowBase, acc) chunkIdx -> do
-                        (nextBase, chunkRows) <- collectChunkRows resPtr columns chunkIdx rowBase
-                        pure (nextBase, chunkRows : acc)
-                    )
-                    (0, [])
-                    [0 .. chunkCount - 1]
-            pure (concat (reverse chunks))
+    collectChunks columns []
+  where
+    collectChunks columns acc = do
+        chunk <- c_duckdb_fetch_chunk resPtr
+        if chunk == nullPtr
+            then pure (concat (reverse acc))
+            else do
+                rows <-
+                    finally
+                        (decodeChunk columns chunk)
+                        (destroyDataChunk chunk)
+                let acc' = maybe acc (: acc) rows
+                collectChunks columns acc'
+
+    decodeChunk columns chunk = do
+        rawSize <- c_duckdb_data_chunk_get_size chunk
+        let rowCount = fromIntegral rawSize :: Int
+        if rowCount <= 0
+            then pure Nothing
+            else
+                if null columns
+                    then pure (Just (replicate rowCount []))
+                    else do
+                        vectors <- prepareChunkVectors chunk columns
+                        rows <- mapM (buildMaterializedRow columns vectors) [0 .. rowCount - 1]
+                        pure (Just rows)
 
 collectResultColumns :: Ptr DuckDBResult -> IO [StatementStreamColumn]
 collectResultColumns resPtr = do
@@ -1046,32 +1071,6 @@ collectResultColumns resPtr = do
                 , statementStreamColumnType = dtype
                 }
 
-collectChunkRows :: Ptr DuckDBResult -> [StatementStreamColumn] -> Int -> Int -> IO (Int, [[Field]])
-collectChunkRows resPtr columns chunkIndex rowBase = do
-    chunk <- c_duckdb_result_get_chunk resPtr (fromIntegral chunkIndex)
-    if chunk == nullPtr
-        then pure (rowBase, [])
-        else
-            finally
-                (do
-                    rawSize <- c_duckdb_data_chunk_get_size chunk
-                    let rowCount = fromIntegral rawSize :: Int
-                    when (rowCount > 0 && not (null columns)) $
-                        void $
-                            fetchFieldValue
-                                resPtr
-                                (statementStreamColumnType (head columns))
-                                (statementStreamColumnIndex (head columns))
-                                rowBase
-                    if rowCount <= 0
-                        then pure (rowBase, [])
-                        else do
-                            vectors <- prepareChunkVectors chunk columns
-                            rows <- mapM (buildMaterializedRow columns vectors) [0 .. rowCount - 1]
-                            pure (rowBase + rowCount, rows)
-                )
-                (destroyDataChunk chunk)
-
 buildMaterializedRow :: [StatementStreamColumn] -> [StatementStreamChunkVector] -> Int -> IO [Field]
 buildMaterializedRow columns vectors rowIdx =
     zipWithM (buildMaterializedField rowIdx) columns vectors
@@ -1092,40 +1091,86 @@ buildMaterializedField rowIdx column StatementStreamChunkVector{statementStreamC
             , fieldValue = value
             }
 
-fetchFieldValue :: Ptr DuckDBResult -> DuckDBType -> Int -> Int -> IO FieldValue
-fetchFieldValue resPtr dtype columnIndex rowIndex =
-    withChunkForRow resPtr rowIndex \chunk localRow -> do
-        vector <- c_duckdb_data_chunk_get_vector chunk (fromIntegral columnIndex)
-        dataPtr <- c_duckdb_vector_get_data vector
-        validity <- c_duckdb_vector_get_validity vector
-        materializedValueFromPointers dtype vector dataPtr validity localRow
+data DuckDBListEntry = DuckDBListEntry
+    { duckDBListEntryOffset :: !Word64
+    , duckDBListEntryLength :: !Word64
+    }
+    deriving (Eq, Show)
 
-withChunkForRow :: Ptr DuckDBResult -> Int -> (DuckDBDataChunk -> Int -> IO a) -> IO a
-withChunkForRow resPtr targetRow action = do
-    chunkCount <- (fromIntegral <$> c_duckdb_result_chunk_count resPtr) :: IO Int
-    let seek chunkIdx remaining
-            | chunkIdx >= chunkCount =
-                throwIO (userError "duckdb-simple: row index out of bounds while materialising result")
-            | otherwise = do
-                chunk <- c_duckdb_result_get_chunk resPtr (fromIntegral chunkIdx)
-                if chunk == nullPtr
-                    then seek (chunkIdx + 1) remaining
-                    else do
-                        rawSize <- c_duckdb_data_chunk_get_size chunk
-                        let rowCount = fromIntegral rawSize :: Int
-                        if rowCount <= 0
-                            then do
-                                destroyDataChunk chunk
-                                seek (chunkIdx + 1) remaining
-                            else if remaining < rowCount
-                                then
-                                    finally
-                                        (action chunk remaining)
-                                        (destroyDataChunk chunk)
-                                else do
-                                    destroyDataChunk chunk
-                                    seek (chunkIdx + 1) (remaining - rowCount)
-    seek 0 targetRow
+instance Storable DuckDBListEntry where
+    sizeOf _ = listEntryWordSize * 2
+    alignment _ = alignment (0 :: Word64)
+    peek ptr = do
+        offset <- peekByteOff ptr 0
+        len <- peekByteOff ptr listEntryWordSize
+        pure DuckDBListEntry{duckDBListEntryOffset = offset, duckDBListEntryLength = len}
+    poke ptr DuckDBListEntry{duckDBListEntryOffset, duckDBListEntryLength} = do
+        pokeByteOff ptr 0 duckDBListEntryOffset
+        pokeByteOff ptr listEntryWordSize duckDBListEntryLength
+
+listEntryWordSize :: Int
+listEntryWordSize = sizeOf (0 :: Word64)
+
+decodeListElements :: DuckDBVector -> Ptr () -> Int -> IO [FieldValue]
+decodeListElements vector dataPtr rowIdx = do
+    entry <- peekElemOff (castPtr dataPtr :: Ptr DuckDBListEntry) rowIdx
+    (baseIdx, len) <- listEntryBounds (Text.pack "list") entry
+    childVec <- c_duckdb_list_vector_get_child vector
+    when (childVec == nullPtr) $
+        throwIO (userError "duckdb-simple: list child vector is null")
+    childType <- vectorElementType childVec
+    childData <- c_duckdb_vector_get_data childVec
+    childValidity <- c_duckdb_vector_get_validity childVec
+    forM [0 .. len - 1] \delta ->
+        materializedValueFromPointers childType childVec childData childValidity (baseIdx + delta)
+
+decodeMapPairs :: DuckDBVector -> Ptr () -> Int -> IO [(FieldValue, FieldValue)]
+decodeMapPairs vector dataPtr rowIdx = do
+    entry <- peekElemOff (castPtr dataPtr :: Ptr DuckDBListEntry) rowIdx
+    (baseIdx, len) <- listEntryBounds (Text.pack "map") entry
+    structVec <- c_duckdb_list_vector_get_child vector
+    when (structVec == nullPtr) $
+        throwIO (userError "duckdb-simple: map struct vector is null")
+    keyVec <- c_duckdb_struct_vector_get_child structVec 0
+    valueVec <- c_duckdb_struct_vector_get_child structVec 1
+    when (keyVec == nullPtr || valueVec == nullPtr) $
+        throwIO (userError "duckdb-simple: map child vectors are null")
+    keyType <- vectorElementType keyVec
+    valueType <- vectorElementType valueVec
+    keyData <- c_duckdb_vector_get_data keyVec
+    valueData <- c_duckdb_vector_get_data valueVec
+    keyValidity <- c_duckdb_vector_get_validity keyVec
+    valueValidity <- c_duckdb_vector_get_validity valueVec
+    forM [0 .. len - 1] \delta -> do
+        let childIdx = baseIdx + delta
+        keyValue <- materializedValueFromPointers keyType keyVec keyData keyValidity childIdx
+        valueValue <- materializedValueFromPointers valueType valueVec valueData valueValidity childIdx
+        pure (keyValue, valueValue)
+
+vectorElementType :: DuckDBVector -> IO DuckDBType
+vectorElementType vec = do
+    logical <- c_duckdb_vector_get_column_type vec
+    dtype <- c_duckdb_get_type_id logical
+    destroyLogicalType logical
+    pure dtype
+
+listEntryBounds :: Text -> DuckDBListEntry -> IO (Int, Int)
+listEntryBounds context DuckDBListEntry{duckDBListEntryOffset, duckDBListEntryLength} = do
+    base <- ensureWithinIntRange (context <> Text.pack " offset") duckDBListEntryOffset
+    len <- ensureWithinIntRange (context <> Text.pack " length") duckDBListEntryLength
+    let maxInt = toInteger (maxBound :: Int)
+        upperBound = toInteger base + toInteger len - 1
+    when (len > 0 && upperBound > maxInt) $
+        throwIO (userError ("duckdb-simple: " <> Text.unpack context <> " bounds exceed Int range"))
+    pure (base, len)
+
+ensureWithinIntRange :: Text -> Word64 -> IO Int
+ensureWithinIntRange context value =
+    let actual = toInteger value
+        limit = toInteger (maxBound :: Int)
+     in if actual <= limit
+            then pure (fromInteger actual)
+            else throwIO (userError ("duckdb-simple: " <> Text.unpack context <> " exceeds Int range"))
 
 materializedValueFromPointers :: DuckDBType -> DuckDBVector -> Ptr () -> Ptr Word64 -> Int -> IO FieldValue
 materializedValueFromPointers dtype vector dataPtr validity rowIdx = do
@@ -1134,23 +1179,6 @@ materializedValueFromPointers dtype vector dataPtr validity rowIdx = do
     if not valid
         then pure FieldNull
         else do
-            (decimalInfo, enumInternal) <-
-                if dtype == DuckDBTypeDecimal || dtype == DuckDBTypeEnum
-                    then do
-                        logical <- c_duckdb_vector_get_column_type vector
-                        info <-
-                            case dtype of
-                                DuckDBTypeDecimal -> do
-                                    width <- c_duckdb_decimal_width logical
-                                    scale <- c_duckdb_decimal_scale logical
-                                    pure (Just (width, scale), Nothing)
-                                DuckDBTypeEnum -> do
-                                    internal <- c_duckdb_enum_internal_type logical
-                                    pure (Nothing, Just internal)
-                                _ -> pure (Nothing, Nothing)
-                        destroyLogicalType logical
-                        pure info
-                    else pure (Nothing, Nothing)
             case dtype of
                 DuckDBTypeBoolean -> do
                     raw <- peekElemOff (castPtr dataPtr :: Ptr Word8) rowIdx
@@ -1186,7 +1214,11 @@ materializedValueFromPointers dtype vector dataPtr validity rowIdx = do
                     raw <- peekElemOff (castPtr dataPtr :: Ptr Double) rowIdx
                     pure (FieldDouble raw)
                 DuckDBTypeVarchar -> FieldText <$> chunkDecodeText dataPtr duckIdx
-                DuckDBTypeUUID -> FieldText <$> chunkDecodeText dataPtr duckIdx
+                DuckDBTypeUUID -> do
+                    DuckDBUHugeInt lower upper_biased <-
+                      peekElemOff (castPtr dataPtr :: Ptr DuckDBUHugeInt) rowIdx
+                    let upper = upper_biased `xor` (0x8000000000000000 :: Word64)
+                    return $ FieldUUID $ UUID.fromWords64 (fromIntegral upper) lower
                 DuckDBTypeBlob -> FieldBlob <$> chunkDecodeBlob dataPtr duckIdx
                 DuckDBTypeDate -> do
                     raw <- peekElemOff (castPtr dataPtr :: Ptr Int32) rowIdx
@@ -1219,13 +1251,24 @@ materializedValueFromPointers dtype vector dataPtr validity rowIdx = do
                     raw <- peekElemOff (castPtr dataPtr :: Ptr DuckDBInterval) rowIdx
                     pure (FieldInterval (intervalValueFromDuckDB raw))
                 DuckDBTypeDecimal -> do
-                    raw <- peekElemOff (castPtr dataPtr :: Ptr DuckDBDecimal) rowIdx
-                    case decimalInfo of
-                        Just (width, scale) -> do
-                            decimal <- decimalValueFromDuckDB width scale raw
-                            pure (FieldDecimal decimal)
-                        Nothing ->
-                            error "duckdb-simple: missing decimal metadata for result vector"
+                    bracket (c_duckdb_vector_get_column_type vector)
+                            (\lty -> alloca $ \ptr -> poke ptr lty >> c_duckdb_destroy_logical_type ptr) $
+                              \logical -> do
+                                 width <- c_duckdb_decimal_width logical
+                                 scale <- c_duckdb_decimal_scale logical
+                                 internal_ty <- c_duckdb_decimal_internal_type logical
+                                 raw <- case internal_ty of
+                                     DuckDBTypeSmallInt ->
+                                         toInteger <$> peekElemOff (castPtr dataPtr :: Ptr Int16) rowIdx
+                                     DuckDBTypeInteger ->
+                                         toInteger <$> peekElemOff (castPtr dataPtr :: Ptr Int32) rowIdx
+                                     DuckDBTypeBigInt ->
+                                         toInteger <$> peekElemOff (castPtr dataPtr :: Ptr Int64) rowIdx
+                                     DuckDBTypeHugeInt ->
+                                         toInteger . duckDBHugeIntToInteger <$>
+                                            peekElemOff (castPtr dataPtr :: Ptr DuckDBHugeInt) rowIdx
+                                     _ -> error "duckdb-simple: unsupported decimal internal storage type"
+                                 pure (FieldDecimal (DecimalValue width scale raw))
                 DuckDBTypeHugeInt -> do
                     raw <- peekElemOff (castPtr dataPtr :: Ptr DuckDBHugeInt) rowIdx
                     pure (FieldHugeInt (duckDBHugeIntToInteger raw))
@@ -1233,24 +1276,52 @@ materializedValueFromPointers dtype vector dataPtr validity rowIdx = do
                     raw <- peekElemOff (castPtr dataPtr :: Ptr DuckDBUHugeInt) rowIdx
                     pure (FieldUHugeInt (duckDBUHugeIntToInteger raw))
                 DuckDBTypeBit -> do
-                    raw <- peekElemOff (castPtr dataPtr :: Ptr DuckDBBit) rowIdx
-                    FieldBit <$> decodeDuckDBBit raw
+                    -- Same deal as with bignum, we need to manually decode the string_t
+                    let base = castPtr dataPtr :: Ptr Word8
+                        -- luckily, the underlying data has only one field, which is a string_t
+                        offset = fromIntegral rowIdx * duckdbStringTSize
+                        stringPtr = castPtr (base `plusPtr` offset) :: Ptr DuckDBStringT
+                    len <- c_duckdb_string_t_length stringPtr
+                    ptr <- c_duckdb_string_t_data stringPtr
+                    bs <- BS.unpack <$> BS.packCStringLen (ptr, fromIntegral len)
+                    case bs of
+                       [] -> return $ FieldBit (BitString 0 BS.empty)
+                       [padding] -> return $ FieldBit (BitString padding BS.empty)
+                       (padding:b:bits) -> do
+                           let tc = [(8-fromIntegral padding)..7]
+                               cleared = foldl clearBit b tc
+                           return $ FieldBit $ BitString padding (BS.pack (cleared:bits))
                 DuckDBTypeBigNum -> do
-                    raw <- peekElemOff (castPtr dataPtr :: Ptr DuckDBBignum) rowIdx
-                    FieldBigNum <$> decodeDuckDBBigNum raw
+                    -- Here we have to get messy, because DuckDB returns a pointer
+                    -- to a bignum_t and not a duckdb_bignum.
+                    let base = castPtr dataPtr :: Ptr Word8
+                        -- luckily, the underlying data has only one field, which is a string_t
+                        offset = fromIntegral rowIdx * duckdbStringTSize
+                        stringPtr = castPtr (base `plusPtr` offset) :: Ptr DuckDBStringT
+                    len <- c_duckdb_string_t_length stringPtr
+                    if len < 3
+                    then return $ FieldBigNum (BigNum 0)
+                    else do ptr <- c_duckdb_string_t_data stringPtr
+                            bs <- BS.unpack <$> BS.packCStringLen (ptr, fromIntegral len)
+                            return $ FieldBigNum $ BigNum (fromBigNumBytes bs)
+                DuckDBTypeList -> FieldList <$> decodeListElements vector dataPtr rowIdx
+                DuckDBTypeMap -> FieldMap <$> decodeMapPairs vector dataPtr rowIdx
+                DuckDBTypeStruct -> error "duckdb-simple: STRUCT columns are not supported"
+                DuckDBTypeUnion -> error "duckdb-simple: UNION columns are not supported"
                 DuckDBTypeEnum -> do
-                    case enumInternal of
-                        Just DuckDBTypeUTinyInt -> do
-                            raw <- peekElemOff (castPtr dataPtr :: Ptr Word8) rowIdx
-                            pure (FieldEnum (fromIntegral raw))
-                        Just DuckDBTypeUSmallInt -> do
-                            raw <- peekElemOff (castPtr dataPtr :: Ptr Word16) rowIdx
-                            pure (FieldEnum (fromIntegral raw))
-                        Just DuckDBTypeUInteger -> do
-                            raw <- peekElemOff (castPtr dataPtr :: Ptr Word32) rowIdx
-                            pure (FieldEnum raw)
-                        _ ->
-                            error "duckdb-simple: unsupported enum internal storage type"
+                    bracket (c_duckdb_vector_get_column_type vector)
+                            (\lty -> alloca $ \ptr -> poke ptr lty >> c_duckdb_destroy_logical_type ptr) $
+                            \logical -> do
+                              enumInternal <- c_duckdb_enum_internal_type logical
+                              FieldEnum <$>
+                                case enumInternal of
+                                    DuckDBTypeUTinyInt -> do
+                                        fromIntegral <$> peekElemOff (castPtr dataPtr :: Ptr Word8) rowIdx
+                                    DuckDBTypeUSmallInt -> do
+                                        fromIntegral <$> peekElemOff (castPtr dataPtr :: Ptr Word16) rowIdx
+                                    DuckDBTypeUInteger -> do
+                                        fromIntegral <$> peekElemOff (castPtr dataPtr :: Ptr Word32) rowIdx
+                                    _ -> error "duckdb-simple: unsupported enum internal storage type"
                 DuckDBTypeSQLNull ->
                     pure FieldNull
                 DuckDBTypeStringLiteral -> FieldText <$> chunkDecodeText dataPtr duckIdx
@@ -1347,19 +1418,6 @@ intervalValueFromDuckDB DuckDBInterval{duckDBIntervalMonths, duckDBIntervalDays,
         , intervalMicros = duckDBIntervalMicros
         }
 
-decimalValueFromDuckDB :: Word8 -> Word8 -> DuckDBDecimal -> IO DecimalValue
-decimalValueFromDuckDB width scale rawDecimal =
-    alloca \ptr -> do
-        let decimal = rawDecimal{duckDBDecimalWidth = width, duckDBDecimalScale = scale}
-        poke ptr decimal
-        CDouble approx <- c_duckdb_decimal_to_double ptr
-        pure
-            DecimalValue
-                { decimalWidth = width
-                , decimalScale = scale
-                , decimalInteger = duckDBHugeIntToInteger (duckDBDecimalValue decimal)
-                , decimalApproximate = realToFrac approx
-                }
 
 duckDBHugeIntToInteger :: DuckDBHugeInt -> Integer
 duckDBHugeIntToInteger DuckDBHugeInt{duckDBHugeIntLower, duckDBHugeIntUpper} =
@@ -1369,25 +1427,6 @@ duckDBUHugeIntToInteger :: DuckDBUHugeInt -> Integer
 duckDBUHugeIntToInteger DuckDBUHugeInt{duckDBUHugeIntLower, duckDBUHugeIntUpper} =
     (fromIntegral duckDBUHugeIntUpper `shiftL` 64) .|. fromIntegral duckDBUHugeIntLower
 
-decodeDuckDBBit :: DuckDBBit -> IO BitString
-decodeDuckDBBit DuckDBBit{duckDBBitData, duckDBBitSize}
-    | duckDBBitData == nullPtr || duckDBBitSize == 0 =
-        pure BitString{bitStringLength = 0, bitStringBytes = BS.empty}
-    | otherwise = do
-        let bitLength = duckDBBitSize
-            byteLength = fromIntegral ((bitLength + 7) `div` 8) :: Int
-        bytes <- BS.packCStringLen (castPtr duckDBBitData, byteLength)
-        pure BitString{bitStringLength = bitLength, bitStringBytes = bytes}
-
-decodeDuckDBBigNum :: DuckDBBignum -> IO BigNum
-decodeDuckDBBigNum DuckDBBignum{duckDBBignumData, duckDBBignumSize, duckDBBignumIsNegative}
-    | duckDBBignumData == nullPtr || duckDBBignumSize == 0 =
-        pure BigNum{bigNumIsNegative = duckDBBignumIsNegative /= 0, bigNumMagnitude = BS.empty}
-    | otherwise = do
-        let byteLength = fromIntegral duckDBBignumSize :: Int
-        bytes <- BS.packCStringLen (castPtr duckDBBignumData, byteLength)
-        pure BigNum{bigNumIsNegative = duckDBBignumIsNegative /= 0, bigNumMagnitude = bytes}
-
 peekError :: Ptr CString -> IO Text
 peekError ptr = do
     errPtr <- peek ptr
@@ -1400,4 +1439,4 @@ peekError ptr = do
 maybeFreeErr :: Ptr CString -> IO ()
 maybeFreeErr ptr = do
     errPtr <- peek ptr
-    when (errPtr /= nullPtr) $ c_duckdb_free (castPtr errPtr)
+    when (errPtr /= nullPtr) $ c_duckdb_free (castPtr errPtr)
\ No newline at end of file
