@@ -31,51 +31,52 @@ import Control.Exception (
     try,
  )
 import Control.Monad (forM, forM_, when)
+import Data.Bits (shiftL, (.|.), clearBit)
 import qualified Data.ByteString as BS
-import Data.Bits ((.|.), shiftL)
 import Data.Int (Int16, Int32, Int64, Int8)
-import Data.Ratio ((%))
 import Data.Proxy (Proxy (..))
+import Data.Ratio ((%))
 import Data.Text (Text)
 import qualified Data.Text as Text
 import qualified Data.Text.Encoding as TE
 import qualified Data.Text.Foreign as TextForeign
-import Data.Word (Word16, Word32, Word64, Word8)
 import Data.Time.Calendar (Day, fromGregorian)
 import Data.Time.Clock (UTCTime (..))
 import Data.Time.Clock.POSIX (posixSecondsToUTCTime)
-import Data.Time.LocalTime
-    ( LocalTime (..)
-    , TimeOfDay (..)
-    , minutesToTimeZone
-    , utc
-    , utcToLocalTime
-    )
+import Data.Time.LocalTime (
+    LocalTime (..),
+    TimeOfDay (..),
+    minutesToTimeZone,
+    utc,
+    utcToLocalTime,
+ )
+import Data.Word (Word16, Word32, Word64, Word8)
 import Database.DuckDB.FFI
-import Database.DuckDB.Simple.FromField
-    ( BigNum (..)
-    , BitString (..)
-    , DecimalValue (..)
-    , Field (..)
-    , FieldValue (..)
-    , FromField (..)
-    , IntervalValue (..)
-    , TimeWithZone (..)
-    )
-import Database.DuckDB.Simple.Internal
-    ( Connection
-    , Query (..)
-    , SQLError (..)
-    , withConnectionHandle
-    , withQueryCString
-    )
+import Database.DuckDB.Simple.FromField (
+    BigNum (..),
+    BitString (..),
+    DecimalValue (..),
+    Field (..),
+    FieldValue (..),
+    FromField (..),
+    IntervalValue (..),
+    TimeWithZone (..),
+    fromBigNumBytes,
+ )
+import Database.DuckDB.Simple.Internal (
+    Connection,
+    Query (..),
+    SQLError (..),
+    withConnectionHandle,
+    withQueryCString,
+ )
+import Database.DuckDB.Simple.Ok (Ok (..))
 import Foreign.C.String (peekCString)
-import Foreign.C.Types (CBool (..), CDouble (..))
+import Foreign.C.Types (CBool (..))
 import Foreign.Marshal.Alloc (alloca)
 import Foreign.Ptr (Ptr, castPtr, freeHaskellFunPtr, nullPtr, plusPtr)
 import Foreign.StablePtr (StablePtr, castPtrToStablePtr, castStablePtrToPtr, deRefStablePtr, freeStablePtr, newStablePtr)
 import Foreign.Storable (peek, peekElemOff, poke, pokeElemOff)
-import Database.DuckDB.Simple.Ok (Ok(..))
 
 -- | Tag DuckDB logical types we support for scalar return values.
 data ScalarType
@@ -502,11 +503,32 @@ fetchValue dtype decimalInfo enumInternal dataPtr rowIdx =
                             functionInvocationError $
                                 Text.pack "duckdb-simple: missing decimal metadata for scalar function argument"
             DuckDBTypeBit -> do
-                value <- peekElemOff (castPtr dataPtr :: Ptr DuckDBBit) idx
-                FieldBit <$> decodeDuckDBBit value
+                -- Same deal as with bignum, we need to manually decode the string_t
+                let base = castPtr dataPtr :: Ptr Word8
+                    -- luckily, the underlying data has only one field, which is a string_t
+                    offset = fromIntegral rowIdx * duckdbStringTSize
+                    stringPtr = castPtr (base `plusPtr` offset) :: Ptr DuckDBStringT
+                len <- c_duckdb_string_t_length stringPtr
+                ptr <- c_duckdb_string_t_data stringPtr
+                bs <- BS.unpack <$> BS.packCStringLen (ptr, fromIntegral len)
+                case bs of
+                    [] -> return $ FieldBit (BitString 0 BS.empty)
+                    [padding] -> return $ FieldBit (BitString padding BS.empty)
+                    (padding:b:bits) ->
+                        let cleared = foldl clearBit b [0..fromIntegral padding -1]
+                        in return $ FieldBit $ BitString padding (BS.pack (cleared:bits))
             DuckDBTypeBigNum -> do
-                value <- peekElemOff (castPtr dataPtr :: Ptr DuckDBBignum) idx
-                FieldBigNum <$> decodeDuckDBBigNum value
+                let base = castPtr dataPtr :: Ptr Word8
+                    -- luckily, the underlying data has only one field, which is a string_t
+                    offset = fromIntegral rowIdx * duckdbStringTSize
+                    stringPtr = castPtr (base `plusPtr` offset) :: Ptr DuckDBStringT
+                len <- c_duckdb_string_t_length stringPtr
+                if len < 3
+                    then return $ FieldBigNum (BigNum 0)
+                    else do
+                        ptr <- c_duckdb_string_t_data stringPtr
+                        bs <- BS.unpack <$> BS.packCStringLen (ptr, fromIntegral len)
+                        return $ FieldBigNum $ BigNum (fromBigNumBytes bs)
             DuckDBTypeEnum -> do
                 case enumInternal of
                     Just DuckDBTypeUTinyInt -> do
@@ -652,18 +674,14 @@ intervalValueFromDuckDB DuckDBInterval{duckDBIntervalMonths, duckDBIntervalDays,
         }
 
 decimalValueFromDuckDB :: Word8 -> Word8 -> DuckDBDecimal -> IO DecimalValue
-decimalValueFromDuckDB width scale rawDecimal =
-    alloca \ptr -> do
-        let decimal = rawDecimal{duckDBDecimalWidth = width, duckDBDecimalScale = scale}
-        poke ptr decimal
-        CDouble approx <- c_duckdb_decimal_to_double ptr
-        pure
-            DecimalValue
-                { decimalWidth = width
-                , decimalScale = scale
-                , decimalInteger = duckDBHugeIntToInteger (duckDBDecimalValue decimal)
-                , decimalApproximate = realToFrac approx
-                }
+decimalValueFromDuckDB width scale rawDecimal = do
+    let decimal = rawDecimal{duckDBDecimalWidth = width, duckDBDecimalScale = scale}
+    pure
+        DecimalValue
+            { decimalWidth = width
+            , decimalScale = scale
+            , decimalInteger = duckDBHugeIntToInteger (duckDBDecimalValue decimal)
+            }
 
 duckDBHugeIntToInteger :: DuckDBHugeInt -> Integer
 duckDBHugeIntToInteger DuckDBHugeInt{duckDBHugeIntLower, duckDBHugeIntUpper} =
@@ -673,25 +691,6 @@ duckDBUHugeIntToInteger :: DuckDBUHugeInt -> Integer
 duckDBUHugeIntToInteger DuckDBUHugeInt{duckDBUHugeIntLower, duckDBUHugeIntUpper} =
     (fromIntegral duckDBUHugeIntUpper `shiftL` 64) .|. fromIntegral duckDBUHugeIntLower
 
-decodeDuckDBBit :: DuckDBBit -> IO BitString
-decodeDuckDBBit DuckDBBit{duckDBBitData, duckDBBitSize}
-    | duckDBBitData == nullPtr || duckDBBitSize == 0 =
-        pure BitString{bitStringLength = 0, bitStringBytes = BS.empty}
-    | otherwise = do
-        let bitLength = duckDBBitSize
-            byteLength = fromIntegral ((bitLength + 7) `div` 8) :: Int
-        bytes <- BS.packCStringLen (castPtr duckDBBitData, byteLength)
-        pure BitString{bitStringLength = bitLength, bitStringBytes = bytes}
-
-decodeDuckDBBigNum :: DuckDBBignum -> IO BigNum
-decodeDuckDBBigNum DuckDBBignum{duckDBBignumData, duckDBBignumSize, duckDBBignumIsNegative}
-    | duckDBBignumData == nullPtr || duckDBBignumSize == 0 =
-        pure BigNum{bigNumIsNegative = duckDBBignumIsNegative /= 0, bigNumMagnitude = BS.empty}
-    | otherwise = do
-        let byteLength = fromIntegral duckDBBignumSize :: Int
-        bytes <- BS.packCStringLen (castPtr duckDBBignumData, byteLength)
-        pure BigNum{bigNumIsNegative = duckDBBignumIsNegative /= 0, bigNumMagnitude = bytes}
-
 writeResults :: ScalarType -> [ScalarValue] -> DuckDBVector -> IO ()
 writeResults resultType values outVec = do
     let hasNulls = any isNullValue values
