@@ -1,8 +1,10 @@
 {-# LANGUAGE BlockArguments #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE GADTs #-}
+{-# LANGUAGE InstanceSigs #-}
 {-# LANGUAGE NamedFieldPuns #-}
 {-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 
 {- |
 Module      : Database.DuckDB.Simple.ToField
@@ -14,15 +16,19 @@ delegating to the DuckDB C API under the hood.
 module Database.DuckDB.Simple.ToField (
     FieldBinding,
     ToField (..),
+    DuckDBColumnType (..),
     NamedParam (..),
+    duckdbColumnType,
     bindFieldBinding,
     renderFieldBinding,
 ) where
 
 import Control.Exception (bracket, throwIO)
 import Control.Monad (when)
+import Data.Bits (complement)
 import qualified Data.ByteString as BS
 import Data.Int (Int16, Int32, Int64)
+import Data.Proxy (Proxy (..))
 import Data.Text (Text)
 import qualified Data.Text as Text
 import qualified Data.Text.Foreign as TextForeign
@@ -31,6 +37,7 @@ import Data.Time.Clock (UTCTime (..), diffTimeToPicoseconds)
 import Data.Time.LocalTime (LocalTime (..), TimeOfDay (..), timeOfDayToTime, utc, utcToLocalTime)
 import Data.Word (Word16, Word32, Word64, Word8)
 import Database.DuckDB.FFI
+import Database.DuckDB.Simple.FromField (BigNum (..), toBigNumBytes, BitString(..))
 import Database.DuckDB.Simple.Internal (
     SQLError (..),
     Statement (..),
@@ -39,9 +46,12 @@ import Database.DuckDB.Simple.Internal (
 import Database.DuckDB.Simple.Types (Null (..))
 import Foreign.C.String (peekCString)
 import Foreign.C.Types (CDouble (..))
+import Foreign.Marshal (fromBool)
 import Foreign.Marshal.Alloc (alloca)
 import Foreign.Ptr (Ptr, castPtr, nullPtr)
 import Foreign.Storable (poke)
+import Numeric.Natural (Natural)
+import qualified Data.UUID as UUID
 
 -- | Represents a named parameter binding using the @:=@ operator.
 data NamedParam where
@@ -55,6 +65,14 @@ data FieldBinding = FieldBinding
     , fieldBindingDisplay :: !String
     }
 
+-- | Types that map to a concrete DuckDB column type when used with 'ToField'.
+class DuckDBColumnType a where
+    duckdbColumnTypeFor :: Proxy a -> Text
+
+-- | Report the DuckDB column type that best matches a given 'ToField' instance.
+duckdbColumnType :: forall a. (DuckDBColumnType a) => Proxy a -> Text
+duckdbColumnType = duckdbColumnTypeFor
+
 -- | Apply a 'FieldBinding' to the given statement/index.
 bindFieldBinding :: Statement -> DuckDBIdx -> FieldBinding -> IO ()
 bindFieldBinding stmt idx FieldBinding{fieldBindingAction} = fieldBindingAction stmt idx
@@ -71,12 +89,13 @@ mkFieldBinding display action =
         }
 
 -- | Types that can be used as positional parameters.
-class ToField a where
+class (DuckDBColumnType a) => ToField a where
     toField :: a -> FieldBinding
 
 instance ToField Null where
     toField Null = nullBinding "NULL"
 
+
 instance ToField Bool where
     toField value =
         mkFieldBinding (show value) $ \stmt idx ->
@@ -94,20 +113,33 @@ instance ToField Int32 where
 instance ToField Int64 where
     toField = intBinding
 
+instance ToField BigNum where
+    toField = bignumBinding
+
+instance ToField UUID.UUID where
+    toField = uuidBinding
+
+instance ToField Integer where
+    toField :: Integer -> FieldBinding
+    toField = toField . BigNum
+
+instance ToField Natural where
+    toField = toField . BigNum . toInteger
+
 instance ToField Word where
     toField value = uint64Binding (fromIntegral value)
 
 instance ToField Word16 where
-    toField value = uint16Binding value
+    toField = uint16Binding
 
 instance ToField Word32 where
-    toField value = uint32Binding value
+    toField = uint32Binding
 
 instance ToField Word64 where
-    toField value = uint64Binding value
+    toField = uint64Binding
 
 instance ToField Word8 where
-    toField value = uint8Binding value
+    toField = uint8Binding
 
 instance ToField Double where
     toField value =
@@ -139,6 +171,12 @@ instance ToField String where
                 TextForeign.withCString (Text.pack str) $ \cstr ->
                     bindDuckValue stmt idx (c_duckdb_create_varchar cstr)
 
+instance DuckDBColumnType BitString where
+    duckdbColumnTypeFor _ = "BIT"
+
+instance ToField BitString where
+    toField = bitBinding
+
 instance ToField BS.ByteString where
     toField bs =
         mkFieldBinding
@@ -190,6 +228,81 @@ instance (ToField a) => ToField (Maybe a) where
                 { fieldBindingDisplay = "Just " <> renderFieldBinding binding
                 }
 
+instance DuckDBColumnType Null where
+    duckdbColumnTypeFor _ = "NULL"
+
+instance DuckDBColumnType Bool where
+    duckdbColumnTypeFor _ = "BOOLEAN"
+
+instance DuckDBColumnType Int where
+    duckdbColumnTypeFor _ = "BIGINT"
+
+instance DuckDBColumnType Int16 where
+    duckdbColumnTypeFor _ = "SMALLINT"
+
+instance DuckDBColumnType Int32 where
+    duckdbColumnTypeFor _ = "INTEGER"
+
+instance DuckDBColumnType Int64 where
+    duckdbColumnTypeFor _ = "BIGINT"
+
+instance DuckDBColumnType BigNum where
+    duckdbColumnTypeFor _ = "BIGNUM"
+
+instance DuckDBColumnType UUID.UUID where
+    duckdbColumnTypeFor _ = "UUID"
+
+instance DuckDBColumnType Integer where
+    duckdbColumnTypeFor _ = "BIGNUM"
+
+instance DuckDBColumnType Natural where
+    duckdbColumnTypeFor _ = "BIGNUM"
+
+instance DuckDBColumnType Word where
+    duckdbColumnTypeFor _ = "UBIGINT"
+
+instance DuckDBColumnType Word8 where
+    duckdbColumnTypeFor _ = "UTINYINT"
+
+instance DuckDBColumnType Word16 where
+    duckdbColumnTypeFor _ = "USMALLINT"
+
+instance DuckDBColumnType Word32 where
+    duckdbColumnTypeFor _ = "UINTEGER"
+
+instance DuckDBColumnType Word64 where
+    duckdbColumnTypeFor _ = "UBIGINT"
+
+instance DuckDBColumnType Double where
+    duckdbColumnTypeFor _ = "DOUBLE"
+
+instance DuckDBColumnType Float where
+    duckdbColumnTypeFor _ = "FLOAT"
+
+instance DuckDBColumnType Text where
+    duckdbColumnTypeFor _ = "TEXT"
+
+instance DuckDBColumnType String where
+    duckdbColumnTypeFor _ = "TEXT"
+
+instance DuckDBColumnType BS.ByteString where
+    duckdbColumnTypeFor _ = "BLOB"
+
+instance DuckDBColumnType Day where
+    duckdbColumnTypeFor _ = "DATE"
+
+instance DuckDBColumnType TimeOfDay where
+    duckdbColumnTypeFor _ = "TIME"
+
+instance DuckDBColumnType LocalTime where
+    duckdbColumnTypeFor _ = "TIMESTAMP"
+
+instance DuckDBColumnType UTCTime where
+    duckdbColumnTypeFor _ = "TIMESTAMPTZ"
+
+instance (DuckDBColumnType a) => DuckDBColumnType (Maybe a) where
+    duckdbColumnTypeFor _ = duckdbColumnTypeFor (Proxy :: Proxy a)
+
 -- | Helper for binding 'Null' values.
 nullBinding :: String -> FieldBinding
 nullBinding repr =
@@ -233,6 +346,80 @@ uint8Binding value =
         \stmt idx ->
             bindDuckValue stmt idx (c_duckdb_create_uint8 value)
 
+uuidBinding :: UUID.UUID -> FieldBinding
+uuidBinding uuid =
+    mkFieldBinding
+        (show uuid)
+        \stmt idx ->
+            bindDuckValue stmt idx $
+              alloca $ \ptr -> do
+                let (upper, lower) = UUID.toWords64 uuid
+                poke ptr DuckDBUHugeInt
+                    { duckDBUHugeIntLower = lower
+                    , duckDBUHugeIntUpper = upper
+                    }
+                c_duckdb_create_uuid ptr
+
+bitBinding :: BitString -> FieldBinding
+bitBinding (BitString padding bits) =
+    mkFieldBinding
+        (show (BitString padding bits))
+        \stmt idx ->
+            let w_padding = (fromIntegral padding :: Word8):BS.unpack bits
+            in bindDuckValue stmt idx $
+                  if BS.null bits
+                        then alloca \ptr -> do
+                            poke
+                                ptr
+                                DuckDBBit
+                                    { duckDBBitData = nullPtr
+                                    , duckDBBitSize = 0
+                                    }
+                            c_duckdb_create_bit ptr
+                        else
+                            BS.useAsCStringLen (BS.pack w_padding) \(rawPtr, len) ->
+                            alloca \ptr -> do
+                                poke
+                                    ptr
+                                    DuckDBBit
+                                        { duckDBBitData = castPtr rawPtr
+                                        , duckDBBitSize = fromIntegral len
+                                        }
+                                c_duckdb_create_bit ptr
+
+bignumBinding :: BigNum -> FieldBinding
+bignumBinding (BigNum big) =
+    mkFieldBinding
+        (show big)
+        \stmt idx ->
+            bindDuckValue stmt idx $
+                let neg = fromBool (big < 0)
+                    big_num_bytes =
+                        BS.pack $
+                            if big < 0
+                                then map complement (drop 3 $ toBigNumBytes big)
+                                else drop 3 $ toBigNumBytes big
+                 in if BS.null big_num_bytes
+                        then alloca \ptr -> do
+                            poke
+                                ptr
+                                DuckDBBignum
+                                    { duckDBBignumData = nullPtr
+                                    , duckDBBignumSize = 0
+                                    , duckDBBignumIsNegative = neg
+                                    }
+                            c_duckdb_create_bignum ptr
+                        else BS.useAsCStringLen big_num_bytes \(rawPtr, len) ->
+                            alloca \ptr -> do
+                                poke
+                                    ptr
+                                    DuckDBBignum
+                                        { duckDBBignumData = castPtr rawPtr
+                                        , duckDBBignumSize = fromIntegral len
+                                        , duckDBBignumIsNegative = neg
+                                        }
+                                c_duckdb_create_bignum ptr
+
 encodeDay :: Day -> IO DuckDBDate
 encodeDay day =
     alloca \ptr -> do
